# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .history import (
    HistoryResource,
    AsyncHistoryResource,
    HistoryResourceWithRawResponse,
    AsyncHistoryResourceWithRawResponse,
    HistoryResourceWithStreamingResponse,
    AsyncHistoryResourceWithStreamingResponse,
)
from .policies import (
    PoliciesResource,
    AsyncPoliciesResource,
    PoliciesResourceWithRawResponse,
    AsyncPoliciesResourceWithRawResponse,
    PoliciesResourceWithStreamingResponse,
    AsyncPoliciesResourceWithStreamingResponse,
)
from .silences import (
    SilencesResource,
    AsyncSilencesResource,
    SilencesResourceWithRawResponse,
    AsyncSilencesResourceWithRawResponse,
    SilencesResourceWithStreamingResponse,
    AsyncSilencesResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .available_alerts import (
    AvailableAlertsResource,
    AsyncAvailableAlertsResource,
    AvailableAlertsResourceWithRawResponse,
    AsyncAvailableAlertsResourceWithRawResponse,
    AvailableAlertsResourceWithStreamingResponse,
    AsyncAvailableAlertsResourceWithStreamingResponse,
)
from .destinations.destinations import (
    DestinationsResource,
    AsyncDestinationsResource,
    DestinationsResourceWithRawResponse,
    AsyncDestinationsResourceWithRawResponse,
    DestinationsResourceWithStreamingResponse,
    AsyncDestinationsResourceWithStreamingResponse,
)

__all__ = ["AlertingResource", "AsyncAlertingResource"]


class AlertingResource(SyncAPIResource):
    @cached_property
    def available_alerts(self) -> AvailableAlertsResource:
        return AvailableAlertsResource(self._client)

    @cached_property
    def destinations(self) -> DestinationsResource:
        return DestinationsResource(self._client)

    @cached_property
    def history(self) -> HistoryResource:
        return HistoryResource(self._client)

    @cached_property
    def policies(self) -> PoliciesResource:
        return PoliciesResource(self._client)

    @cached_property
    def silences(self) -> SilencesResource:
        return SilencesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AlertingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AlertingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AlertingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AlertingResourceWithStreamingResponse(self)


class AsyncAlertingResource(AsyncAPIResource):
    @cached_property
    def available_alerts(self) -> AsyncAvailableAlertsResource:
        return AsyncAvailableAlertsResource(self._client)

    @cached_property
    def destinations(self) -> AsyncDestinationsResource:
        return AsyncDestinationsResource(self._client)

    @cached_property
    def history(self) -> AsyncHistoryResource:
        return AsyncHistoryResource(self._client)

    @cached_property
    def policies(self) -> AsyncPoliciesResource:
        return AsyncPoliciesResource(self._client)

    @cached_property
    def silences(self) -> AsyncSilencesResource:
        return AsyncSilencesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAlertingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAlertingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAlertingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAlertingResourceWithStreamingResponse(self)


class AlertingResourceWithRawResponse:
    def __init__(self, alerting: AlertingResource) -> None:
        self._alerting = alerting

    @cached_property
    def available_alerts(self) -> AvailableAlertsResourceWithRawResponse:
        return AvailableAlertsResourceWithRawResponse(self._alerting.available_alerts)

    @cached_property
    def destinations(self) -> DestinationsResourceWithRawResponse:
        return DestinationsResourceWithRawResponse(self._alerting.destinations)

    @cached_property
    def history(self) -> HistoryResourceWithRawResponse:
        return HistoryResourceWithRawResponse(self._alerting.history)

    @cached_property
    def policies(self) -> PoliciesResourceWithRawResponse:
        return PoliciesResourceWithRawResponse(self._alerting.policies)

    @cached_property
    def silences(self) -> SilencesResourceWithRawResponse:
        return SilencesResourceWithRawResponse(self._alerting.silences)


class AsyncAlertingResourceWithRawResponse:
    def __init__(self, alerting: AsyncAlertingResource) -> None:
        self._alerting = alerting

    @cached_property
    def available_alerts(self) -> AsyncAvailableAlertsResourceWithRawResponse:
        return AsyncAvailableAlertsResourceWithRawResponse(self._alerting.available_alerts)

    @cached_property
    def destinations(self) -> AsyncDestinationsResourceWithRawResponse:
        return AsyncDestinationsResourceWithRawResponse(self._alerting.destinations)

    @cached_property
    def history(self) -> AsyncHistoryResourceWithRawResponse:
        return AsyncHistoryResourceWithRawResponse(self._alerting.history)

    @cached_property
    def policies(self) -> AsyncPoliciesResourceWithRawResponse:
        return AsyncPoliciesResourceWithRawResponse(self._alerting.policies)

    @cached_property
    def silences(self) -> AsyncSilencesResourceWithRawResponse:
        return AsyncSilencesResourceWithRawResponse(self._alerting.silences)


class AlertingResourceWithStreamingResponse:
    def __init__(self, alerting: AlertingResource) -> None:
        self._alerting = alerting

    @cached_property
    def available_alerts(self) -> AvailableAlertsResourceWithStreamingResponse:
        return AvailableAlertsResourceWithStreamingResponse(self._alerting.available_alerts)

    @cached_property
    def destinations(self) -> DestinationsResourceWithStreamingResponse:
        return DestinationsResourceWithStreamingResponse(self._alerting.destinations)

    @cached_property
    def history(self) -> HistoryResourceWithStreamingResponse:
        return HistoryResourceWithStreamingResponse(self._alerting.history)

    @cached_property
    def policies(self) -> PoliciesResourceWithStreamingResponse:
        return PoliciesResourceWithStreamingResponse(self._alerting.policies)

    @cached_property
    def silences(self) -> SilencesResourceWithStreamingResponse:
        return SilencesResourceWithStreamingResponse(self._alerting.silences)


class AsyncAlertingResourceWithStreamingResponse:
    def __init__(self, alerting: AsyncAlertingResource) -> None:
        self._alerting = alerting

    @cached_property
    def available_alerts(self) -> AsyncAvailableAlertsResourceWithStreamingResponse:
        return AsyncAvailableAlertsResourceWithStreamingResponse(self._alerting.available_alerts)

    @cached_property
    def destinations(self) -> AsyncDestinationsResourceWithStreamingResponse:
        return AsyncDestinationsResourceWithStreamingResponse(self._alerting.destinations)

    @cached_property
    def history(self) -> AsyncHistoryResourceWithStreamingResponse:
        return AsyncHistoryResourceWithStreamingResponse(self._alerting.history)

    @cached_property
    def policies(self) -> AsyncPoliciesResourceWithStreamingResponse:
        return AsyncPoliciesResourceWithStreamingResponse(self._alerting.policies)

    @cached_property
    def silences(self) -> AsyncSilencesResourceWithStreamingResponse:
        return AsyncSilencesResourceWithStreamingResponse(self._alerting.silences)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .traceroutes import (
    TraceroutesResource,
    AsyncTraceroutesResource,
    TraceroutesResourceWithRawResponse,
    AsyncTraceroutesResourceWithRawResponse,
    TraceroutesResourceWithStreamingResponse,
    AsyncTraceroutesResourceWithStreamingResponse,
)
from .endpoint_healthchecks import (
    EndpointHealthchecksResource,
    AsyncEndpointHealthchecksResource,
    EndpointHealthchecksResourceWithRawResponse,
    AsyncEndpointHealthchecksResourceWithRawResponse,
    EndpointHealthchecksResourceWithStreamingResponse,
    AsyncEndpointHealthchecksResourceWithStreamingResponse,
)

__all__ = ["DiagnosticsResource", "AsyncDiagnosticsResource"]


class DiagnosticsResource(SyncAPIResource):
    @cached_property
    def traceroutes(self) -> TraceroutesResource:
        return TraceroutesResource(self._client)

    @cached_property
    def endpoint_healthchecks(self) -> EndpointHealthchecksResource:
        return EndpointHealthchecksResource(self._client)

    @cached_property
    def with_raw_response(self) -> DiagnosticsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DiagnosticsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DiagnosticsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DiagnosticsResourceWithStreamingResponse(self)


class AsyncDiagnosticsResource(AsyncAPIResource):
    @cached_property
    def traceroutes(self) -> AsyncTraceroutesResource:
        return AsyncTraceroutesResource(self._client)

    @cached_property
    def endpoint_healthchecks(self) -> AsyncEndpointHealthchecksResource:
        return AsyncEndpointHealthchecksResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDiagnosticsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDiagnosticsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDiagnosticsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDiagnosticsResourceWithStreamingResponse(self)


class DiagnosticsResourceWithRawResponse:
    def __init__(self, diagnostics: DiagnosticsResource) -> None:
        self._diagnostics = diagnostics

    @cached_property
    def traceroutes(self) -> TraceroutesResourceWithRawResponse:
        return TraceroutesResourceWithRawResponse(self._diagnostics.traceroutes)

    @cached_property
    def endpoint_healthchecks(self) -> EndpointHealthchecksResourceWithRawResponse:
        return EndpointHealthchecksResourceWithRawResponse(self._diagnostics.endpoint_healthchecks)


class AsyncDiagnosticsResourceWithRawResponse:
    def __init__(self, diagnostics: AsyncDiagnosticsResource) -> None:
        self._diagnostics = diagnostics

    @cached_property
    def traceroutes(self) -> AsyncTraceroutesResourceWithRawResponse:
        return AsyncTraceroutesResourceWithRawResponse(self._diagnostics.traceroutes)

    @cached_property
    def endpoint_healthchecks(self) -> AsyncEndpointHealthchecksResourceWithRawResponse:
        return AsyncEndpointHealthchecksResourceWithRawResponse(self._diagnostics.endpoint_healthchecks)


class DiagnosticsResourceWithStreamingResponse:
    def __init__(self, diagnostics: DiagnosticsResource) -> None:
        self._diagnostics = diagnostics

    @cached_property
    def traceroutes(self) -> TraceroutesResourceWithStreamingResponse:
        return TraceroutesResourceWithStreamingResponse(self._diagnostics.traceroutes)

    @cached_property
    def endpoint_healthchecks(self) -> EndpointHealthchecksResourceWithStreamingResponse:
        return EndpointHealthchecksResourceWithStreamingResponse(self._diagnostics.endpoint_healthchecks)


class AsyncDiagnosticsResourceWithStreamingResponse:
    def __init__(self, diagnostics: AsyncDiagnosticsResource) -> None:
        self._diagnostics = diagnostics

    @cached_property
    def traceroutes(self) -> AsyncTraceroutesResourceWithStreamingResponse:
        return AsyncTraceroutesResourceWithStreamingResponse(self._diagnostics.traceroutes)

    @cached_property
    def endpoint_healthchecks(self) -> AsyncEndpointHealthchecksResourceWithStreamingResponse:
        return AsyncEndpointHealthchecksResourceWithStreamingResponse(self._diagnostics.endpoint_healthchecks)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from .....pagination import SyncSinglePage, AsyncSinglePage
from .scripts.scripts import (
    ScriptsResource,
    AsyncScriptsResource,
    ScriptsResourceWithRawResponse,
    AsyncScriptsResourceWithRawResponse,
    ScriptsResourceWithStreamingResponse,
    AsyncScriptsResourceWithStreamingResponse,
)
from ....._base_client import AsyncPaginator, make_request_options
from .....types.workers_for_platforms.dispatch import namespace_create_params
from .....types.workers_for_platforms.dispatch.namespace_get_response import NamespaceGetResponse
from .....types.workers_for_platforms.dispatch.namespace_list_response import NamespaceListResponse
from .....types.workers_for_platforms.dispatch.namespace_create_response import NamespaceCreateResponse

__all__ = ["NamespacesResource", "AsyncNamespacesResource"]


class NamespacesResource(SyncAPIResource):
    @cached_property
    def scripts(self) -> ScriptsResource:
        return ScriptsResource(self._client)

    @cached_property
    def with_raw_response(self) -> NamespacesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return NamespacesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> NamespacesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return NamespacesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[NamespaceCreateResponse]:
        """
        Create a new Workers for Platforms namespace.

        Args:
          account_id: Identifier.

          name: The name of the dispatch namespace.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/workers/dispatch/namespaces", account_id=account_id),
            body=maybe_transform({"name": name}, namespace_create_params.NamespaceCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[NamespaceCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[NamespaceCreateResponse]], ResultWrapper[NamespaceCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[NamespaceListResponse]:
        """
        Fetch a list of Workers for Platforms namespaces.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/workers/dispatch/namespaces", account_id=account_id),
            page=SyncSinglePage[NamespaceListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=NamespaceListResponse,
        )

    def delete(
        self,
        dispatch_namespace: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete a Workers for Platforms namespace.

        Args:
          account_id: Identifier.

          dispatch_namespace: Name of the Workers for Platforms dispatch namespace.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dispatch_namespace:
            raise ValueError(f"Expected a non-empty value for `dispatch_namespace` but received {dispatch_namespace!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/workers/dispatch/namespaces/{dispatch_namespace}",
                account_id=account_id,
                dispatch_namespace=dispatch_namespace,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def get(
        self,
        dispatch_namespace: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[NamespaceGetResponse]:
        """
        Get a Workers for Platforms namespace.

        Args:
          account_id: Identifier.

          dispatch_namespace: Name of the Workers for Platforms dispatch namespace.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dispatch_namespace:
            raise ValueError(f"Expected a non-empty value for `dispatch_namespace` but received {dispatch_namespace!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/workers/dispatch/namespaces/{dispatch_namespace}",
                account_id=account_id,
                dispatch_namespace=dispatch_namespace,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[NamespaceGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[NamespaceGetResponse]], ResultWrapper[NamespaceGetResponse]),
        )


class AsyncNamespacesResource(AsyncAPIResource):
    @cached_property
    def scripts(self) -> AsyncScriptsResource:
        return AsyncScriptsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncNamespacesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncNamespacesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncNamespacesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncNamespacesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[NamespaceCreateResponse]:
        """
        Create a new Workers for Platforms namespace.

        Args:
          account_id: Identifier.

          name: The name of the dispatch namespace.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/workers/dispatch/namespaces", account_id=account_id),
            body=await async_maybe_transform({"name": name}, namespace_create_params.NamespaceCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[NamespaceCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[NamespaceCreateResponse]], ResultWrapper[NamespaceCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[NamespaceListResponse, AsyncSinglePage[NamespaceListResponse]]:
        """
        Fetch a list of Workers for Platforms namespaces.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/workers/dispatch/namespaces", account_id=account_id),
            page=AsyncSinglePage[NamespaceListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=NamespaceListResponse,
        )

    async def delete(
        self,
        dispatch_namespace: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete a Workers for Platforms namespace.

        Args:
          account_id: Identifier.

          dispatch_namespace: Name of the Workers for Platforms dispatch namespace.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dispatch_namespace:
            raise ValueError(f"Expected a non-empty value for `dispatch_namespace` but received {dispatch_namespace!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/workers/dispatch/namespaces/{dispatch_namespace}",
                account_id=account_id,
                dispatch_namespace=dispatch_namespace,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def get(
        self,
        dispatch_namespace: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[NamespaceGetResponse]:
        """
        Get a Workers for Platforms namespace.

        Args:
          account_id: Identifier.

          dispatch_namespace: Name of the Workers for Platforms dispatch namespace.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dispatch_namespace:
            raise ValueError(f"Expected a non-empty value for `dispatch_namespace` but received {dispatch_namespace!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/workers/dispatch/namespaces/{dispatch_namespace}",
                account_id=account_id,
                dispatch_namespace=dispatch_namespace,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[NamespaceGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[NamespaceGetResponse]], ResultWrapper[NamespaceGetResponse]),
        )


class NamespacesResourceWithRawResponse:
    def __init__(self, namespaces: NamespacesResource) -> None:
        self._namespaces = namespaces

        self.create = to_raw_response_wrapper(
            namespaces.create,
        )
        self.list = to_raw_response_wrapper(
            namespaces.list,
        )
        self.delete = to_raw_response_wrapper(
            namespaces.delete,
        )
        self.get = to_raw_response_wrapper(
            namespaces.get,
        )

    @cached_property
    def scripts(self) -> ScriptsResourceWithRawResponse:
        return ScriptsResourceWithRawResponse(self._namespaces.scripts)


class AsyncNamespacesResourceWithRawResponse:
    def __init__(self, namespaces: AsyncNamespacesResource) -> None:
        self._namespaces = namespaces

        self.create = async_to_raw_response_wrapper(
            namespaces.create,
        )
        self.list = async_to_raw_response_wrapper(
            namespaces.list,
        )
        self.delete = async_to_raw_response_wrapper(
            namespaces.delete,
        )
        self.get = async_to_raw_response_wrapper(
            namespaces.get,
        )

    @cached_property
    def scripts(self) -> AsyncScriptsResourceWithRawResponse:
        return AsyncScriptsResourceWithRawResponse(self._namespaces.scripts)


class NamespacesResourceWithStreamingResponse:
    def __init__(self, namespaces: NamespacesResource) -> None:
        self._namespaces = namespaces

        self.create = to_streamed_response_wrapper(
            namespaces.create,
        )
        self.list = to_streamed_response_wrapper(
            namespaces.list,
        )
        self.delete = to_streamed_response_wrapper(
            namespaces.delete,
        )
        self.get = to_streamed_response_wrapper(
            namespaces.get,
        )

    @cached_property
    def scripts(self) -> ScriptsResourceWithStreamingResponse:
        return ScriptsResourceWithStreamingResponse(self._namespaces.scripts)


class AsyncNamespacesResourceWithStreamingResponse:
    def __init__(self, namespaces: AsyncNamespacesResource) -> None:
        self._namespaces = namespaces

        self.create = async_to_streamed_response_wrapper(
            namespaces.create,
        )
        self.list = async_to_streamed_response_wrapper(
            namespaces.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            namespaces.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            namespaces.get,
        )

    @cached_property
    def scripts(self) -> AsyncScriptsResourceWithStreamingResponse:
        return AsyncScriptsResourceWithStreamingResponse(self._namespaces.scripts)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from .devices import (
    DevicesResource,
    AsyncDevicesResource,
    DevicesResourceWithRawResponse,
    AsyncDevicesResourceWithRawResponse,
    DevicesResourceWithStreamingResponse,
    AsyncDevicesResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.zero_trust.dex import fleet_status_live_params, fleet_status_over_time_params
from .....types.zero_trust.dex.fleet_status_live_response import FleetStatusLiveResponse
from .....types.zero_trust.dex.fleet_status_over_time_response import FleetStatusOverTimeResponse

__all__ = ["FleetStatusResource", "AsyncFleetStatusResource"]


class FleetStatusResource(SyncAPIResource):
    @cached_property
    def devices(self) -> DevicesResource:
        return DevicesResource(self._client)

    @cached_property
    def with_raw_response(self) -> FleetStatusResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return FleetStatusResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FleetStatusResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return FleetStatusResourceWithStreamingResponse(self)

    def live(
        self,
        *,
        account_id: str,
        since_minutes: float,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FleetStatusLiveResponse]:
        """
        List details for live (up to 60 minutes) devices using WARP

        Args:
          since_minutes: Number of minutes before current time

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/dex/fleet-status/live", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"since_minutes": since_minutes}, fleet_status_live_params.FleetStatusLiveParams),
                post_parser=ResultWrapper[Optional[FleetStatusLiveResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FleetStatusLiveResponse]], ResultWrapper[FleetStatusLiveResponse]),
        )

    def over_time(
        self,
        *,
        account_id: str,
        from_: str,
        to: str,
        colo: str | Omit = omit,
        device_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FleetStatusOverTimeResponse]:
        """
        List details for devices using WARP, up to 7 days

        Args:
          from_: Time range beginning in ISO format

          to: Time range end in ISO format

          colo: Cloudflare colo

          device_id: Device-specific ID, given as UUID v4

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/dex/fleet-status/over-time", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "from_": from_,
                        "to": to,
                        "colo": colo,
                        "device_id": device_id,
                    },
                    fleet_status_over_time_params.FleetStatusOverTimeParams,
                ),
                post_parser=ResultWrapper[Optional[FleetStatusOverTimeResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FleetStatusOverTimeResponse]], ResultWrapper[FleetStatusOverTimeResponse]),
        )


class AsyncFleetStatusResource(AsyncAPIResource):
    @cached_property
    def devices(self) -> AsyncDevicesResource:
        return AsyncDevicesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncFleetStatusResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFleetStatusResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFleetStatusResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncFleetStatusResourceWithStreamingResponse(self)

    async def live(
        self,
        *,
        account_id: str,
        since_minutes: float,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FleetStatusLiveResponse]:
        """
        List details for live (up to 60 minutes) devices using WARP

        Args:
          since_minutes: Number of minutes before current time

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/dex/fleet-status/live", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"since_minutes": since_minutes}, fleet_status_live_params.FleetStatusLiveParams
                ),
                post_parser=ResultWrapper[Optional[FleetStatusLiveResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FleetStatusLiveResponse]], ResultWrapper[FleetStatusLiveResponse]),
        )

    async def over_time(
        self,
        *,
        account_id: str,
        from_: str,
        to: str,
        colo: str | Omit = omit,
        device_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FleetStatusOverTimeResponse]:
        """
        List details for devices using WARP, up to 7 days

        Args:
          from_: Time range beginning in ISO format

          to: Time range end in ISO format

          colo: Cloudflare colo

          device_id: Device-specific ID, given as UUID v4

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/dex/fleet-status/over-time", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "from_": from_,
                        "to": to,
                        "colo": colo,
                        "device_id": device_id,
                    },
                    fleet_status_over_time_params.FleetStatusOverTimeParams,
                ),
                post_parser=ResultWrapper[Optional[FleetStatusOverTimeResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FleetStatusOverTimeResponse]], ResultWrapper[FleetStatusOverTimeResponse]),
        )


class FleetStatusResourceWithRawResponse:
    def __init__(self, fleet_status: FleetStatusResource) -> None:
        self._fleet_status = fleet_status

        self.live = to_raw_response_wrapper(
            fleet_status.live,
        )
        self.over_time = to_raw_response_wrapper(
            fleet_status.over_time,
        )

    @cached_property
    def devices(self) -> DevicesResourceWithRawResponse:
        return DevicesResourceWithRawResponse(self._fleet_status.devices)


class AsyncFleetStatusResourceWithRawResponse:
    def __init__(self, fleet_status: AsyncFleetStatusResource) -> None:
        self._fleet_status = fleet_status

        self.live = async_to_raw_response_wrapper(
            fleet_status.live,
        )
        self.over_time = async_to_raw_response_wrapper(
            fleet_status.over_time,
        )

    @cached_property
    def devices(self) -> AsyncDevicesResourceWithRawResponse:
        return AsyncDevicesResourceWithRawResponse(self._fleet_status.devices)


class FleetStatusResourceWithStreamingResponse:
    def __init__(self, fleet_status: FleetStatusResource) -> None:
        self._fleet_status = fleet_status

        self.live = to_streamed_response_wrapper(
            fleet_status.live,
        )
        self.over_time = to_streamed_response_wrapper(
            fleet_status.over_time,
        )

    @cached_property
    def devices(self) -> DevicesResourceWithStreamingResponse:
        return DevicesResourceWithStreamingResponse(self._fleet_status.devices)


class AsyncFleetStatusResourceWithStreamingResponse:
    def __init__(self, fleet_status: AsyncFleetStatusResource) -> None:
        self._fleet_status = fleet_status

        self.live = async_to_streamed_response_wrapper(
            fleet_status.live,
        )
        self.over_time = async_to_streamed_response_wrapper(
            fleet_status.over_time,
        )

    @cached_property
    def devices(self) -> AsyncDevicesResourceWithStreamingResponse:
        return AsyncDevicesResourceWithStreamingResponse(self._fleet_status.devices)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["Decision"]

Decision: TypeAlias = Literal["allow", "deny", "non_identity", "bypass"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ....._utils import PropertyInfo

__all__ = ["ManagedUpdateParams"]


class ManagedUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    enabled: Required[bool]
    """Whether to enable public bucket access at the r2.dev domain."""

    jurisdiction: Annotated[Literal["default", "eu", "fedramp"], PropertyInfo(alias="cf-r2-jurisdiction")]
    """Jurisdiction where objects in this bucket are guaranteed to be stored."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["Scope"]


class Scope(BaseModel):
    """Used only for ECMP routes."""

    colo_names: Optional[List[str]] = None
    """List of colo names for the ECMP scope."""

    colo_regions: Optional[List[str]] = None
    """List of colo regions for the ECMP scope."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["NamespaceCreateResponse"]


class NamespaceCreateResponse(BaseModel):
    created_by: Optional[str] = None
    """Identifier."""

    created_on: Optional[datetime] = None
    """When the script was created."""

    modified_by: Optional[str] = None
    """Identifier."""

    modified_on: Optional[datetime] = None
    """When the script was last modified."""

    namespace_id: Optional[str] = None
    """API Resource UUID tag."""

    namespace_name: Optional[str] = None
    """Name of the Workers for Platforms dispatch namespace."""

    script_count: Optional[int] = None
    """The current number of scripts in this Dispatch Namespace."""

    trusted_workers: Optional[bool] = None
    """Whether the Workers in the namespace are executed in a "trusted" manner.

    When a Worker is trusted, it has access to the shared caches for the zone in the
    Cache API, and has access to the `request.cf` object on incoming Requests. When
    a Worker is untrusted, caches are not shared across the zone, and `request.cf`
    is undefined. By default, Workers in a namespace are "untrusted".
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .schedule import (
    ScheduleResource,
    AsyncScheduleResource,
    ScheduleResourceWithRawResponse,
    AsyncScheduleResourceWithRawResponse,
    ScheduleResourceWithStreamingResponse,
    AsyncScheduleResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .pages.pages import (
    PagesResource,
    AsyncPagesResource,
    PagesResourceWithRawResponse,
    AsyncPagesResourceWithRawResponse,
    PagesResourceWithStreamingResponse,
    AsyncPagesResourceWithStreamingResponse,
)
from .availabilities import (
    AvailabilitiesResource,
    AsyncAvailabilitiesResource,
    AvailabilitiesResourceWithRawResponse,
    AsyncAvailabilitiesResourceWithRawResponse,
    AvailabilitiesResourceWithStreamingResponse,
    AsyncAvailabilitiesResourceWithStreamingResponse,
)

__all__ = ["SpeedResource", "AsyncSpeedResource"]


class SpeedResource(SyncAPIResource):
    @cached_property
    def schedule(self) -> ScheduleResource:
        return ScheduleResource(self._client)

    @cached_property
    def availabilities(self) -> AvailabilitiesResource:
        return AvailabilitiesResource(self._client)

    @cached_property
    def pages(self) -> PagesResource:
        return PagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> SpeedResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SpeedResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SpeedResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SpeedResourceWithStreamingResponse(self)


class AsyncSpeedResource(AsyncAPIResource):
    @cached_property
    def schedule(self) -> AsyncScheduleResource:
        return AsyncScheduleResource(self._client)

    @cached_property
    def availabilities(self) -> AsyncAvailabilitiesResource:
        return AsyncAvailabilitiesResource(self._client)

    @cached_property
    def pages(self) -> AsyncPagesResource:
        return AsyncPagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSpeedResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSpeedResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSpeedResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSpeedResourceWithStreamingResponse(self)


class SpeedResourceWithRawResponse:
    def __init__(self, speed: SpeedResource) -> None:
        self._speed = speed

    @cached_property
    def schedule(self) -> ScheduleResourceWithRawResponse:
        return ScheduleResourceWithRawResponse(self._speed.schedule)

    @cached_property
    def availabilities(self) -> AvailabilitiesResourceWithRawResponse:
        return AvailabilitiesResourceWithRawResponse(self._speed.availabilities)

    @cached_property
    def pages(self) -> PagesResourceWithRawResponse:
        return PagesResourceWithRawResponse(self._speed.pages)


class AsyncSpeedResourceWithRawResponse:
    def __init__(self, speed: AsyncSpeedResource) -> None:
        self._speed = speed

    @cached_property
    def schedule(self) -> AsyncScheduleResourceWithRawResponse:
        return AsyncScheduleResourceWithRawResponse(self._speed.schedule)

    @cached_property
    def availabilities(self) -> AsyncAvailabilitiesResourceWithRawResponse:
        return AsyncAvailabilitiesResourceWithRawResponse(self._speed.availabilities)

    @cached_property
    def pages(self) -> AsyncPagesResourceWithRawResponse:
        return AsyncPagesResourceWithRawResponse(self._speed.pages)


class SpeedResourceWithStreamingResponse:
    def __init__(self, speed: SpeedResource) -> None:
        self._speed = speed

    @cached_property
    def schedule(self) -> ScheduleResourceWithStreamingResponse:
        return ScheduleResourceWithStreamingResponse(self._speed.schedule)

    @cached_property
    def availabilities(self) -> AvailabilitiesResourceWithStreamingResponse:
        return AvailabilitiesResourceWithStreamingResponse(self._speed.availabilities)

    @cached_property
    def pages(self) -> PagesResourceWithStreamingResponse:
        return PagesResourceWithStreamingResponse(self._speed.pages)


class AsyncSpeedResourceWithStreamingResponse:
    def __init__(self, speed: AsyncSpeedResource) -> None:
        self._speed = speed

    @cached_property
    def schedule(self) -> AsyncScheduleResourceWithStreamingResponse:
        return AsyncScheduleResourceWithStreamingResponse(self._speed.schedule)

    @cached_property
    def availabilities(self) -> AsyncAvailabilitiesResourceWithStreamingResponse:
        return AsyncAvailabilitiesResourceWithStreamingResponse(self._speed.availabilities)

    @cached_property
    def pages(self) -> AsyncPagesResourceWithStreamingResponse:
        return AsyncPagesResourceWithStreamingResponse(self._speed.pages)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["ProtocolDetection"]


class ProtocolDetection(BaseModel):
    """Specify whether to detect protocols from the initial bytes of client traffic."""

    enabled: Optional[bool] = None
    """Specify whether to detect protocols from the initial bytes of client traffic."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .analyze import (
    AnalyzeResource,
    AsyncAnalyzeResource,
    AnalyzeResourceWithRawResponse,
    AsyncAnalyzeResourceWithRawResponse,
    AnalyzeResourceWithStreamingResponse,
    AsyncAnalyzeResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .verification import (
    VerificationResource,
    AsyncVerificationResource,
    VerificationResourceWithRawResponse,
    AsyncVerificationResourceWithRawResponse,
    VerificationResourceWithStreamingResponse,
    AsyncVerificationResourceWithStreamingResponse,
)
from .recommendations import (
    RecommendationsResource,
    AsyncRecommendationsResource,
    RecommendationsResourceWithRawResponse,
    AsyncRecommendationsResourceWithRawResponse,
    RecommendationsResourceWithStreamingResponse,
    AsyncRecommendationsResourceWithStreamingResponse,
)
from .universal.universal import (
    UniversalResource,
    AsyncUniversalResource,
    UniversalResourceWithRawResponse,
    AsyncUniversalResourceWithRawResponse,
    UniversalResourceWithStreamingResponse,
    AsyncUniversalResourceWithStreamingResponse,
)
from .certificate_packs.certificate_packs import (
    CertificatePacksResource,
    AsyncCertificatePacksResource,
    CertificatePacksResourceWithRawResponse,
    AsyncCertificatePacksResourceWithRawResponse,
    CertificatePacksResourceWithStreamingResponse,
    AsyncCertificatePacksResourceWithStreamingResponse,
)

__all__ = ["SSLResource", "AsyncSSLResource"]


class SSLResource(SyncAPIResource):
    @cached_property
    def analyze(self) -> AnalyzeResource:
        return AnalyzeResource(self._client)

    @cached_property
    def certificate_packs(self) -> CertificatePacksResource:
        return CertificatePacksResource(self._client)

    @cached_property
    def recommendations(self) -> RecommendationsResource:
        return RecommendationsResource(self._client)

    @cached_property
    def universal(self) -> UniversalResource:
        return UniversalResource(self._client)

    @cached_property
    def verification(self) -> VerificationResource:
        return VerificationResource(self._client)

    @cached_property
    def with_raw_response(self) -> SSLResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SSLResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SSLResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SSLResourceWithStreamingResponse(self)


class AsyncSSLResource(AsyncAPIResource):
    @cached_property
    def analyze(self) -> AsyncAnalyzeResource:
        return AsyncAnalyzeResource(self._client)

    @cached_property
    def certificate_packs(self) -> AsyncCertificatePacksResource:
        return AsyncCertificatePacksResource(self._client)

    @cached_property
    def recommendations(self) -> AsyncRecommendationsResource:
        return AsyncRecommendationsResource(self._client)

    @cached_property
    def universal(self) -> AsyncUniversalResource:
        return AsyncUniversalResource(self._client)

    @cached_property
    def verification(self) -> AsyncVerificationResource:
        return AsyncVerificationResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSSLResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSSLResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSSLResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSSLResourceWithStreamingResponse(self)


class SSLResourceWithRawResponse:
    def __init__(self, ssl: SSLResource) -> None:
        self._ssl = ssl

    @cached_property
    def analyze(self) -> AnalyzeResourceWithRawResponse:
        return AnalyzeResourceWithRawResponse(self._ssl.analyze)

    @cached_property
    def certificate_packs(self) -> CertificatePacksResourceWithRawResponse:
        return CertificatePacksResourceWithRawResponse(self._ssl.certificate_packs)

    @cached_property
    def recommendations(self) -> RecommendationsResourceWithRawResponse:
        return RecommendationsResourceWithRawResponse(self._ssl.recommendations)

    @cached_property
    def universal(self) -> UniversalResourceWithRawResponse:
        return UniversalResourceWithRawResponse(self._ssl.universal)

    @cached_property
    def verification(self) -> VerificationResourceWithRawResponse:
        return VerificationResourceWithRawResponse(self._ssl.verification)


class AsyncSSLResourceWithRawResponse:
    def __init__(self, ssl: AsyncSSLResource) -> None:
        self._ssl = ssl

    @cached_property
    def analyze(self) -> AsyncAnalyzeResourceWithRawResponse:
        return AsyncAnalyzeResourceWithRawResponse(self._ssl.analyze)

    @cached_property
    def certificate_packs(self) -> AsyncCertificatePacksResourceWithRawResponse:
        return AsyncCertificatePacksResourceWithRawResponse(self._ssl.certificate_packs)

    @cached_property
    def recommendations(self) -> AsyncRecommendationsResourceWithRawResponse:
        return AsyncRecommendationsResourceWithRawResponse(self._ssl.recommendations)

    @cached_property
    def universal(self) -> AsyncUniversalResourceWithRawResponse:
        return AsyncUniversalResourceWithRawResponse(self._ssl.universal)

    @cached_property
    def verification(self) -> AsyncVerificationResourceWithRawResponse:
        return AsyncVerificationResourceWithRawResponse(self._ssl.verification)


class SSLResourceWithStreamingResponse:
    def __init__(self, ssl: SSLResource) -> None:
        self._ssl = ssl

    @cached_property
    def analyze(self) -> AnalyzeResourceWithStreamingResponse:
        return AnalyzeResourceWithStreamingResponse(self._ssl.analyze)

    @cached_property
    def certificate_packs(self) -> CertificatePacksResourceWithStreamingResponse:
        return CertificatePacksResourceWithStreamingResponse(self._ssl.certificate_packs)

    @cached_property
    def recommendations(self) -> RecommendationsResourceWithStreamingResponse:
        return RecommendationsResourceWithStreamingResponse(self._ssl.recommendations)

    @cached_property
    def universal(self) -> UniversalResourceWithStreamingResponse:
        return UniversalResourceWithStreamingResponse(self._ssl.universal)

    @cached_property
    def verification(self) -> VerificationResourceWithStreamingResponse:
        return VerificationResourceWithStreamingResponse(self._ssl.verification)


class AsyncSSLResourceWithStreamingResponse:
    def __init__(self, ssl: AsyncSSLResource) -> None:
        self._ssl = ssl

    @cached_property
    def analyze(self) -> AsyncAnalyzeResourceWithStreamingResponse:
        return AsyncAnalyzeResourceWithStreamingResponse(self._ssl.analyze)

    @cached_property
    def certificate_packs(self) -> AsyncCertificatePacksResourceWithStreamingResponse:
        return AsyncCertificatePacksResourceWithStreamingResponse(self._ssl.certificate_packs)

    @cached_property
    def recommendations(self) -> AsyncRecommendationsResourceWithStreamingResponse:
        return AsyncRecommendationsResourceWithStreamingResponse(self._ssl.recommendations)

    @cached_property
    def universal(self) -> AsyncUniversalResourceWithStreamingResponse:
        return AsyncUniversalResourceWithStreamingResponse(self._ssl.universal)

    @cached_property
    def verification(self) -> AsyncVerificationResourceWithStreamingResponse:
        return AsyncVerificationResourceWithStreamingResponse(self._ssl.verification)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .d1 import D1 as D1
from .query_result import QueryResult as QueryResult
from .database_raw_params import DatabaseRawParams as DatabaseRawParams
from .database_edit_params import DatabaseEditParams as DatabaseEditParams
from .database_list_params import DatabaseListParams as DatabaseListParams
from .database_query_params import DatabaseQueryParams as DatabaseQueryParams
from .database_raw_response import DatabaseRawResponse as DatabaseRawResponse
from .database_create_params import DatabaseCreateParams as DatabaseCreateParams
from .database_export_params import DatabaseExportParams as DatabaseExportParams
from .database_import_params import DatabaseImportParams as DatabaseImportParams
from .database_list_response import DatabaseListResponse as DatabaseListResponse
from .database_update_params import DatabaseUpdateParams as DatabaseUpdateParams
from .database_export_response import DatabaseExportResponse as DatabaseExportResponse
from .database_import_response import DatabaseImportResponse as DatabaseImportResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .cnis import (
    CNIsResource,
    AsyncCNIsResource,
    CNIsResourceWithRawResponse,
    AsyncCNIsResourceWithRawResponse,
    CNIsResourceWithStreamingResponse,
    AsyncCNIsResourceWithStreamingResponse,
)
from .slots import (
    SlotsResource,
    AsyncSlotsResource,
    SlotsResourceWithRawResponse,
    AsyncSlotsResourceWithRawResponse,
    SlotsResourceWithStreamingResponse,
    AsyncSlotsResourceWithStreamingResponse,
)
from .settings import (
    SettingsResource,
    AsyncSettingsResource,
    SettingsResourceWithRawResponse,
    AsyncSettingsResourceWithRawResponse,
    SettingsResourceWithStreamingResponse,
    AsyncSettingsResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .interconnects import (
    InterconnectsResource,
    AsyncInterconnectsResource,
    InterconnectsResourceWithRawResponse,
    AsyncInterconnectsResourceWithRawResponse,
    InterconnectsResourceWithStreamingResponse,
    AsyncInterconnectsResourceWithStreamingResponse,
)

__all__ = ["NetworkInterconnectsResource", "AsyncNetworkInterconnectsResource"]


class NetworkInterconnectsResource(SyncAPIResource):
    @cached_property
    def cnis(self) -> CNIsResource:
        return CNIsResource(self._client)

    @cached_property
    def interconnects(self) -> InterconnectsResource:
        return InterconnectsResource(self._client)

    @cached_property
    def settings(self) -> SettingsResource:
        return SettingsResource(self._client)

    @cached_property
    def slots(self) -> SlotsResource:
        return SlotsResource(self._client)

    @cached_property
    def with_raw_response(self) -> NetworkInterconnectsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return NetworkInterconnectsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> NetworkInterconnectsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return NetworkInterconnectsResourceWithStreamingResponse(self)


class AsyncNetworkInterconnectsResource(AsyncAPIResource):
    @cached_property
    def cnis(self) -> AsyncCNIsResource:
        return AsyncCNIsResource(self._client)

    @cached_property
    def interconnects(self) -> AsyncInterconnectsResource:
        return AsyncInterconnectsResource(self._client)

    @cached_property
    def settings(self) -> AsyncSettingsResource:
        return AsyncSettingsResource(self._client)

    @cached_property
    def slots(self) -> AsyncSlotsResource:
        return AsyncSlotsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncNetworkInterconnectsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncNetworkInterconnectsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncNetworkInterconnectsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncNetworkInterconnectsResourceWithStreamingResponse(self)


class NetworkInterconnectsResourceWithRawResponse:
    def __init__(self, network_interconnects: NetworkInterconnectsResource) -> None:
        self._network_interconnects = network_interconnects

    @cached_property
    def cnis(self) -> CNIsResourceWithRawResponse:
        return CNIsResourceWithRawResponse(self._network_interconnects.cnis)

    @cached_property
    def interconnects(self) -> InterconnectsResourceWithRawResponse:
        return InterconnectsResourceWithRawResponse(self._network_interconnects.interconnects)

    @cached_property
    def settings(self) -> SettingsResourceWithRawResponse:
        return SettingsResourceWithRawResponse(self._network_interconnects.settings)

    @cached_property
    def slots(self) -> SlotsResourceWithRawResponse:
        return SlotsResourceWithRawResponse(self._network_interconnects.slots)


class AsyncNetworkInterconnectsResourceWithRawResponse:
    def __init__(self, network_interconnects: AsyncNetworkInterconnectsResource) -> None:
        self._network_interconnects = network_interconnects

    @cached_property
    def cnis(self) -> AsyncCNIsResourceWithRawResponse:
        return AsyncCNIsResourceWithRawResponse(self._network_interconnects.cnis)

    @cached_property
    def interconnects(self) -> AsyncInterconnectsResourceWithRawResponse:
        return AsyncInterconnectsResourceWithRawResponse(self._network_interconnects.interconnects)

    @cached_property
    def settings(self) -> AsyncSettingsResourceWithRawResponse:
        return AsyncSettingsResourceWithRawResponse(self._network_interconnects.settings)

    @cached_property
    def slots(self) -> AsyncSlotsResourceWithRawResponse:
        return AsyncSlotsResourceWithRawResponse(self._network_interconnects.slots)


class NetworkInterconnectsResourceWithStreamingResponse:
    def __init__(self, network_interconnects: NetworkInterconnectsResource) -> None:
        self._network_interconnects = network_interconnects

    @cached_property
    def cnis(self) -> CNIsResourceWithStreamingResponse:
        return CNIsResourceWithStreamingResponse(self._network_interconnects.cnis)

    @cached_property
    def interconnects(self) -> InterconnectsResourceWithStreamingResponse:
        return InterconnectsResourceWithStreamingResponse(self._network_interconnects.interconnects)

    @cached_property
    def settings(self) -> SettingsResourceWithStreamingResponse:
        return SettingsResourceWithStreamingResponse(self._network_interconnects.settings)

    @cached_property
    def slots(self) -> SlotsResourceWithStreamingResponse:
        return SlotsResourceWithStreamingResponse(self._network_interconnects.slots)


class AsyncNetworkInterconnectsResourceWithStreamingResponse:
    def __init__(self, network_interconnects: AsyncNetworkInterconnectsResource) -> None:
        self._network_interconnects = network_interconnects

    @cached_property
    def cnis(self) -> AsyncCNIsResourceWithStreamingResponse:
        return AsyncCNIsResourceWithStreamingResponse(self._network_interconnects.cnis)

    @cached_property
    def interconnects(self) -> AsyncInterconnectsResourceWithStreamingResponse:
        return AsyncInterconnectsResourceWithStreamingResponse(self._network_interconnects.interconnects)

    @cached_property
    def settings(self) -> AsyncSettingsResourceWithStreamingResponse:
        return AsyncSettingsResourceWithStreamingResponse(self._network_interconnects.settings)

    @cached_property
    def slots(self) -> AsyncSlotsResourceWithStreamingResponse:
        return AsyncSlotsResourceWithStreamingResponse(self._network_interconnects.slots)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["HostnameParam"]


class HostnameParam(TypedDict, total=False):
    """
    Valid characters for hostnames are ASCII(7) letters from a to z, the digits from 0 to 9, wildcards (*), and the hyphen (-).
    """

    url_hostname: Required[str]

    exclude_exact_hostname: bool
    """Only applies to wildcard hostnames (e.g., \\**.example.com).

    When true (default), only subdomains are blocked. When false, both the root
    domain and subdomains are blocked.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .hostnames import (
    HostnamesResource,
    AsyncHostnamesResource,
    HostnamesResourceWithRawResponse,
    AsyncHostnamesResourceWithRawResponse,
    HostnamesResourceWithStreamingResponse,
    AsyncHostnamesResourceWithStreamingResponse,
)
from .ipfs_universal_paths import (
    IPFSUniversalPathsResource,
    AsyncIPFSUniversalPathsResource,
    IPFSUniversalPathsResourceWithRawResponse,
    AsyncIPFSUniversalPathsResourceWithRawResponse,
    IPFSUniversalPathsResourceWithStreamingResponse,
    AsyncIPFSUniversalPathsResourceWithStreamingResponse,
)

__all__ = [
    "IPFSUniversalPathsResource",
    "AsyncIPFSUniversalPathsResource",
    "IPFSUniversalPathsResourceWithRawResponse",
    "AsyncIPFSUniversalPathsResourceWithRawResponse",
    "IPFSUniversalPathsResourceWithStreamingResponse",
    "AsyncIPFSUniversalPathsResourceWithStreamingResponse",
    "HostnamesResource",
    "AsyncHostnamesResource",
    "HostnamesResourceWithRawResponse",
    "AsyncHostnamesResourceWithRawResponse",
    "HostnamesResourceWithStreamingResponse",
    "AsyncHostnamesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from .global_warp_override import (
    GlobalWARPOverrideResource,
    AsyncGlobalWARPOverrideResource,
    GlobalWARPOverrideResourceWithRawResponse,
    AsyncGlobalWARPOverrideResourceWithRawResponse,
    GlobalWARPOverrideResourceWithStreamingResponse,
    AsyncGlobalWARPOverrideResourceWithStreamingResponse,
)

__all__ = ["ResilienceResource", "AsyncResilienceResource"]


class ResilienceResource(SyncAPIResource):
    @cached_property
    def global_warp_override(self) -> GlobalWARPOverrideResource:
        return GlobalWARPOverrideResource(self._client)

    @cached_property
    def with_raw_response(self) -> ResilienceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ResilienceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ResilienceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ResilienceResourceWithStreamingResponse(self)


class AsyncResilienceResource(AsyncAPIResource):
    @cached_property
    def global_warp_override(self) -> AsyncGlobalWARPOverrideResource:
        return AsyncGlobalWARPOverrideResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncResilienceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncResilienceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncResilienceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncResilienceResourceWithStreamingResponse(self)


class ResilienceResourceWithRawResponse:
    def __init__(self, resilience: ResilienceResource) -> None:
        self._resilience = resilience

    @cached_property
    def global_warp_override(self) -> GlobalWARPOverrideResourceWithRawResponse:
        return GlobalWARPOverrideResourceWithRawResponse(self._resilience.global_warp_override)


class AsyncResilienceResourceWithRawResponse:
    def __init__(self, resilience: AsyncResilienceResource) -> None:
        self._resilience = resilience

    @cached_property
    def global_warp_override(self) -> AsyncGlobalWARPOverrideResourceWithRawResponse:
        return AsyncGlobalWARPOverrideResourceWithRawResponse(self._resilience.global_warp_override)


class ResilienceResourceWithStreamingResponse:
    def __init__(self, resilience: ResilienceResource) -> None:
        self._resilience = resilience

    @cached_property
    def global_warp_override(self) -> GlobalWARPOverrideResourceWithStreamingResponse:
        return GlobalWARPOverrideResourceWithStreamingResponse(self._resilience.global_warp_override)


class AsyncResilienceResourceWithStreamingResponse:
    def __init__(self, resilience: AsyncResilienceResource) -> None:
        self._resilience = resilience

    @cached_property
    def global_warp_override(self) -> AsyncGlobalWARPOverrideResourceWithStreamingResponse:
        return AsyncGlobalWARPOverrideResourceWithStreamingResponse(self._resilience.global_warp_override)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["RouteDeleteResponse"]


class RouteDeleteResponse(BaseModel):
    id: Optional[str] = None
    """Identifier."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .origin_ca_certificate import OriginCACertificate as OriginCACertificate
from .origin_ca_certificate_list_params import OriginCACertificateListParams as OriginCACertificateListParams
from .origin_ca_certificate_create_params import OriginCACertificateCreateParams as OriginCACertificateCreateParams
from .origin_ca_certificate_delete_response import (
    OriginCACertificateDeleteResponse as OriginCACertificateDeleteResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["DomainGetResponse", "ValidationData", "VerificationData"]


class ValidationData(BaseModel):
    method: Literal["http", "txt"]

    status: Literal["initializing", "pending", "active", "deactivated", "error"]

    error_message: Optional[str] = None

    txt_name: Optional[str] = None

    txt_value: Optional[str] = None


class VerificationData(BaseModel):
    status: Literal["pending", "active", "deactivated", "blocked", "error"]

    error_message: Optional[str] = None


class DomainGetResponse(BaseModel):
    id: str

    certificate_authority: Literal["google", "lets_encrypt"]

    created_on: str

    domain_id: str

    name: str
    """The domain name."""

    status: Literal["initializing", "pending", "active", "deactivated", "blocked", "error"]

    validation_data: ValidationData

    verification_data: VerificationData

    zone_tag: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["HoldDeleteParams"]


class HoldDeleteParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    hold_after: str
    """
    If `hold_after` is provided, the hold will be temporarily disabled, then
    automatically re-enabled by the system at the time specified in this
    RFC3339-formatted timestamp. Otherwise, the hold will be disabled indefinitely.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, cast

import httpx

from .rules import (
    RulesResource,
    AsyncRulesResource,
    RulesResourceWithRawResponse,
    AsyncRulesResourceWithRawResponse,
    RulesResourceWithStreamingResponse,
    AsyncRulesResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from .versions import (
    VersionsResource,
    AsyncVersionsResource,
    VersionsResourceWithRawResponse,
    AsyncVersionsResourceWithRawResponse,
    VersionsResourceWithStreamingResponse,
    AsyncVersionsResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncCursorPagination, AsyncCursorPagination
from .phases.phases import (
    PhasesResource,
    AsyncPhasesResource,
    PhasesResourceWithRawResponse,
    AsyncPhasesResourceWithRawResponse,
    PhasesResourceWithStreamingResponse,
    AsyncPhasesResourceWithStreamingResponse,
)
from ..._base_client import AsyncPaginator, make_request_options
from ...types.rulesets import Kind, Phase, ruleset_list_params, ruleset_create_params, ruleset_update_params
from ...types.rulesets.kind import Kind
from ...types.rulesets.phase import Phase
from ...types.rulesets.ruleset_get_response import RulesetGetResponse
from ...types.rulesets.ruleset_list_response import RulesetListResponse
from ...types.rulesets.ruleset_create_response import RulesetCreateResponse
from ...types.rulesets.ruleset_update_response import RulesetUpdateResponse

__all__ = ["RulesetsResource", "AsyncRulesetsResource"]


class RulesetsResource(SyncAPIResource):
    @cached_property
    def phases(self) -> PhasesResource:
        return PhasesResource(self._client)

    @cached_property
    def rules(self) -> RulesResource:
        return RulesResource(self._client)

    @cached_property
    def versions(self) -> VersionsResource:
        return VersionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> RulesetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RulesetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RulesetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RulesetsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        kind: Kind,
        name: str,
        phase: Phase,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        description: str | Omit = omit,
        rules: Iterable[ruleset_create_params.Rule] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RulesetCreateResponse:
        """
        Creates a ruleset.

        Args:
          kind: The kind of the ruleset.

          name: The human-readable name of the ruleset.

          phase: The phase of the ruleset.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          description: An informative description of the ruleset.

          rules: The list of rules in the ruleset.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._post(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/rulesets",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=maybe_transform(
                {
                    "kind": kind,
                    "name": name,
                    "phase": phase,
                    "description": description,
                    "rules": rules,
                },
                ruleset_create_params.RulesetCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[RulesetCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[RulesetCreateResponse], ResultWrapper[RulesetCreateResponse]),
        )

    def update(
        self,
        ruleset_id: str,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        description: str | Omit = omit,
        kind: Kind | Omit = omit,
        name: str | Omit = omit,
        phase: Phase | Omit = omit,
        rules: Iterable[ruleset_update_params.Rule] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RulesetUpdateResponse:
        """
        Updates an account or zone ruleset, creating a new version.

        Args:
          ruleset_id: The unique ID of the ruleset.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          description: An informative description of the ruleset.

          kind: The kind of the ruleset.

          name: The human-readable name of the ruleset.

          phase: The phase of the ruleset.

          rules: The list of rules in the ruleset.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not ruleset_id:
            raise ValueError(f"Expected a non-empty value for `ruleset_id` but received {ruleset_id!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._put(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/rulesets/{ruleset_id}",
                ruleset_id=ruleset_id,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=maybe_transform(
                {
                    "description": description,
                    "kind": kind,
                    "name": name,
                    "phase": phase,
                    "rules": rules,
                },
                ruleset_update_params.RulesetUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[RulesetUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[RulesetUpdateResponse], ResultWrapper[RulesetUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        cursor: str | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPagination[RulesetListResponse]:
        """
        Fetches all rulesets.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          cursor: The cursor to use for the next page.

          per_page: The number of rulesets to return per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/rulesets",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=SyncCursorPagination[RulesetListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "per_page": per_page,
                    },
                    ruleset_list_params.RulesetListParams,
                ),
            ),
            model=RulesetListResponse,
        )

    def delete(
        self,
        ruleset_id: str,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes all versions of an existing account or zone ruleset.

        Args:
          ruleset_id: The unique ID of the ruleset.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not ruleset_id:
            raise ValueError(f"Expected a non-empty value for `ruleset_id` but received {ruleset_id!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/rulesets/{ruleset_id}",
                ruleset_id=ruleset_id,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get(
        self,
        ruleset_id: str,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RulesetGetResponse:
        """
        Fetches the latest version of an account or zone ruleset.

        Args:
          ruleset_id: The unique ID of the ruleset.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not ruleset_id:
            raise ValueError(f"Expected a non-empty value for `ruleset_id` but received {ruleset_id!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/rulesets/{ruleset_id}",
                ruleset_id=ruleset_id,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[RulesetGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[RulesetGetResponse], ResultWrapper[RulesetGetResponse]),
        )


class AsyncRulesetsResource(AsyncAPIResource):
    @cached_property
    def phases(self) -> AsyncPhasesResource:
        return AsyncPhasesResource(self._client)

    @cached_property
    def rules(self) -> AsyncRulesResource:
        return AsyncRulesResource(self._client)

    @cached_property
    def versions(self) -> AsyncVersionsResource:
        return AsyncVersionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRulesetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRulesetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRulesetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRulesetsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        kind: Kind,
        name: str,
        phase: Phase,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        description: str | Omit = omit,
        rules: Iterable[ruleset_create_params.Rule] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RulesetCreateResponse:
        """
        Creates a ruleset.

        Args:
          kind: The kind of the ruleset.

          name: The human-readable name of the ruleset.

          phase: The phase of the ruleset.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          description: An informative description of the ruleset.

          rules: The list of rules in the ruleset.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return await self._post(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/rulesets",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=await async_maybe_transform(
                {
                    "kind": kind,
                    "name": name,
                    "phase": phase,
                    "description": description,
                    "rules": rules,
                },
                ruleset_create_params.RulesetCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[RulesetCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[RulesetCreateResponse], ResultWrapper[RulesetCreateResponse]),
        )

    async def update(
        self,
        ruleset_id: str,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        description: str | Omit = omit,
        kind: Kind | Omit = omit,
        name: str | Omit = omit,
        phase: Phase | Omit = omit,
        rules: Iterable[ruleset_update_params.Rule] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RulesetUpdateResponse:
        """
        Updates an account or zone ruleset, creating a new version.

        Args:
          ruleset_id: The unique ID of the ruleset.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          description: An informative description of the ruleset.

          kind: The kind of the ruleset.

          name: The human-readable name of the ruleset.

          phase: The phase of the ruleset.

          rules: The list of rules in the ruleset.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not ruleset_id:
            raise ValueError(f"Expected a non-empty value for `ruleset_id` but received {ruleset_id!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return await self._put(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/rulesets/{ruleset_id}",
                ruleset_id=ruleset_id,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=await async_maybe_transform(
                {
                    "description": description,
                    "kind": kind,
                    "name": name,
                    "phase": phase,
                    "rules": rules,
                },
                ruleset_update_params.RulesetUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[RulesetUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[RulesetUpdateResponse], ResultWrapper[RulesetUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        cursor: str | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[RulesetListResponse, AsyncCursorPagination[RulesetListResponse]]:
        """
        Fetches all rulesets.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          cursor: The cursor to use for the next page.

          per_page: The number of rulesets to return per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/rulesets",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=AsyncCursorPagination[RulesetListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "per_page": per_page,
                    },
                    ruleset_list_params.RulesetListParams,
                ),
            ),
            model=RulesetListResponse,
        )

    async def delete(
        self,
        ruleset_id: str,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes all versions of an existing account or zone ruleset.

        Args:
          ruleset_id: The unique ID of the ruleset.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not ruleset_id:
            raise ValueError(f"Expected a non-empty value for `ruleset_id` but received {ruleset_id!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/rulesets/{ruleset_id}",
                ruleset_id=ruleset_id,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def get(
        self,
        ruleset_id: str,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RulesetGetResponse:
        """
        Fetches the latest version of an account or zone ruleset.

        Args:
          ruleset_id: The unique ID of the ruleset.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not ruleset_id:
            raise ValueError(f"Expected a non-empty value for `ruleset_id` but received {ruleset_id!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return await self._get(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/rulesets/{ruleset_id}",
                ruleset_id=ruleset_id,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[RulesetGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[RulesetGetResponse], ResultWrapper[RulesetGetResponse]),
        )


class RulesetsResourceWithRawResponse:
    def __init__(self, rulesets: RulesetsResource) -> None:
        self._rulesets = rulesets

        self.create = to_raw_response_wrapper(
            rulesets.create,
        )
        self.update = to_raw_response_wrapper(
            rulesets.update,
        )
        self.list = to_raw_response_wrapper(
            rulesets.list,
        )
        self.delete = to_raw_response_wrapper(
            rulesets.delete,
        )
        self.get = to_raw_response_wrapper(
            rulesets.get,
        )

    @cached_property
    def phases(self) -> PhasesResourceWithRawResponse:
        return PhasesResourceWithRawResponse(self._rulesets.phases)

    @cached_property
    def rules(self) -> RulesResourceWithRawResponse:
        return RulesResourceWithRawResponse(self._rulesets.rules)

    @cached_property
    def versions(self) -> VersionsResourceWithRawResponse:
        return VersionsResourceWithRawResponse(self._rulesets.versions)


class AsyncRulesetsResourceWithRawResponse:
    def __init__(self, rulesets: AsyncRulesetsResource) -> None:
        self._rulesets = rulesets

        self.create = async_to_raw_response_wrapper(
            rulesets.create,
        )
        self.update = async_to_raw_response_wrapper(
            rulesets.update,
        )
        self.list = async_to_raw_response_wrapper(
            rulesets.list,
        )
        self.delete = async_to_raw_response_wrapper(
            rulesets.delete,
        )
        self.get = async_to_raw_response_wrapper(
            rulesets.get,
        )

    @cached_property
    def phases(self) -> AsyncPhasesResourceWithRawResponse:
        return AsyncPhasesResourceWithRawResponse(self._rulesets.phases)

    @cached_property
    def rules(self) -> AsyncRulesResourceWithRawResponse:
        return AsyncRulesResourceWithRawResponse(self._rulesets.rules)

    @cached_property
    def versions(self) -> AsyncVersionsResourceWithRawResponse:
        return AsyncVersionsResourceWithRawResponse(self._rulesets.versions)


class RulesetsResourceWithStreamingResponse:
    def __init__(self, rulesets: RulesetsResource) -> None:
        self._rulesets = rulesets

        self.create = to_streamed_response_wrapper(
            rulesets.create,
        )
        self.update = to_streamed_response_wrapper(
            rulesets.update,
        )
        self.list = to_streamed_response_wrapper(
            rulesets.list,
        )
        self.delete = to_streamed_response_wrapper(
            rulesets.delete,
        )
        self.get = to_streamed_response_wrapper(
            rulesets.get,
        )

    @cached_property
    def phases(self) -> PhasesResourceWithStreamingResponse:
        return PhasesResourceWithStreamingResponse(self._rulesets.phases)

    @cached_property
    def rules(self) -> RulesResourceWithStreamingResponse:
        return RulesResourceWithStreamingResponse(self._rulesets.rules)

    @cached_property
    def versions(self) -> VersionsResourceWithStreamingResponse:
        return VersionsResourceWithStreamingResponse(self._rulesets.versions)


class AsyncRulesetsResourceWithStreamingResponse:
    def __init__(self, rulesets: AsyncRulesetsResource) -> None:
        self._rulesets = rulesets

        self.create = async_to_streamed_response_wrapper(
            rulesets.create,
        )
        self.update = async_to_streamed_response_wrapper(
            rulesets.update,
        )
        self.list = async_to_streamed_response_wrapper(
            rulesets.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            rulesets.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            rulesets.get,
        )

    @cached_property
    def phases(self) -> AsyncPhasesResourceWithStreamingResponse:
        return AsyncPhasesResourceWithStreamingResponse(self._rulesets.phases)

    @cached_property
    def rules(self) -> AsyncRulesResourceWithStreamingResponse:
        return AsyncRulesResourceWithStreamingResponse(self._rulesets.rules)

    @cached_property
    def versions(self) -> AsyncVersionsResourceWithStreamingResponse:
        return AsyncVersionsResourceWithStreamingResponse(self._rulesets.versions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .widgets import (
    WidgetsResource,
    AsyncWidgetsResource,
    WidgetsResourceWithRawResponse,
    AsyncWidgetsResourceWithRawResponse,
    WidgetsResourceWithStreamingResponse,
    AsyncWidgetsResourceWithStreamingResponse,
)
from .turnstile import (
    TurnstileResource,
    AsyncTurnstileResource,
    TurnstileResourceWithRawResponse,
    AsyncTurnstileResourceWithRawResponse,
    TurnstileResourceWithStreamingResponse,
    AsyncTurnstileResourceWithStreamingResponse,
)

__all__ = [
    "WidgetsResource",
    "AsyncWidgetsResource",
    "WidgetsResourceWithRawResponse",
    "AsyncWidgetsResourceWithRawResponse",
    "WidgetsResourceWithStreamingResponse",
    "AsyncWidgetsResourceWithStreamingResponse",
    "TurnstileResource",
    "AsyncTurnstileResource",
    "TurnstileResourceWithRawResponse",
    "AsyncTurnstileResourceWithRawResponse",
    "TurnstileResourceWithStreamingResponse",
    "AsyncTurnstileResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .telemetry import (
    TelemetryResource,
    AsyncTelemetryResource,
    TelemetryResourceWithRawResponse,
    AsyncTelemetryResourceWithRawResponse,
    TelemetryResourceWithStreamingResponse,
    AsyncTelemetryResourceWithStreamingResponse,
)
from .destinations import (
    DestinationsResource,
    AsyncDestinationsResource,
    DestinationsResourceWithRawResponse,
    AsyncDestinationsResourceWithRawResponse,
    DestinationsResourceWithStreamingResponse,
    AsyncDestinationsResourceWithStreamingResponse,
)
from .observability import (
    ObservabilityResource,
    AsyncObservabilityResource,
    ObservabilityResourceWithRawResponse,
    AsyncObservabilityResourceWithRawResponse,
    ObservabilityResourceWithStreamingResponse,
    AsyncObservabilityResourceWithStreamingResponse,
)

__all__ = [
    "TelemetryResource",
    "AsyncTelemetryResource",
    "TelemetryResourceWithRawResponse",
    "AsyncTelemetryResourceWithRawResponse",
    "TelemetryResourceWithStreamingResponse",
    "AsyncTelemetryResourceWithStreamingResponse",
    "DestinationsResource",
    "AsyncDestinationsResource",
    "DestinationsResourceWithRawResponse",
    "AsyncDestinationsResourceWithRawResponse",
    "DestinationsResourceWithStreamingResponse",
    "AsyncDestinationsResourceWithStreamingResponse",
    "ObservabilityResource",
    "AsyncObservabilityResource",
    "ObservabilityResourceWithRawResponse",
    "AsyncObservabilityResourceWithRawResponse",
    "ObservabilityResourceWithStreamingResponse",
    "AsyncObservabilityResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr
from .login_design_param import LoginDesignParam

__all__ = ["OrganizationCreateParams", "MfaConfig", "MfaSSHPivKeyRequirements"]


class OrganizationCreateParams(TypedDict, total=False):
    auth_domain: Required[str]
    """The unique subdomain assigned to your Zero Trust organization."""

    name: Required[str]
    """The name of your Zero Trust organization."""

    account_id: str
    """The Account ID to use for this endpoint. Mutually exclusive with the Zone ID."""

    zone_id: str
    """The Zone ID to use for this endpoint. Mutually exclusive with the Account ID."""

    allow_authenticate_via_warp: bool
    """
    When set to true, users can authenticate via WARP for any application in your
    organization. Application settings will take precedence over this value.
    """

    auto_redirect_to_identity: bool
    """
    When set to `true`, users skip the identity provider selection step during
    login.
    """

    deny_unmatched_requests: bool
    """
    Determines whether to deny all requests to Cloudflare-protected resources that
    lack an associated Access application. If enabled, you must explicitly configure
    an Access application and policy to allow traffic to your Cloudflare-protected
    resources. For domains you want to be public across all subdomains, add the
    domain to the `deny_unmatched_requests_exempted_zone_names` array.
    """

    deny_unmatched_requests_exempted_zone_names: SequenceNotStr[str]
    """Contains zone names to exempt from the `deny_unmatched_requests` feature.

    Requests to a subdomain in an exempted zone will block unauthenticated traffic
    by default if there is a configured Access application and policy that matches
    the request.
    """

    is_ui_read_only: bool
    """Lock all settings as Read-Only in the Dashboard, regardless of user permission.

    Updates may only be made via the API or Terraform for this account when enabled.
    """

    login_design: LoginDesignParam

    mfa_config: MfaConfig
    """Configures multi-factor authentication (MFA) settings for an organization."""

    mfa_required_for_all_apps: bool
    """Determines whether global MFA settings apply to applications by default.

    The organization must have MFA enabled with at least one authentication method
    and a session duration configured.
    """

    mfa_ssh_piv_key_requirements: MfaSSHPivKeyRequirements
    """Configures SSH PIV key requirements for MFA using hardware security keys."""

    session_duration: str
    """The amount of time that tokens issued for applications will be valid.

    Must be in the format `300ms` or `2h45m`. Valid time units are: ns, us (or µs),
    ms, s, m, h.
    """

    ui_read_only_toggle_reason: str
    """A description of the reason why the UI read only field is being toggled."""

    user_seat_expiration_inactive_time: str
    """The amount of time a user seat is inactive before it expires.

    When the user seat exceeds the set time of inactivity, the user is removed as an
    active seat and no longer counts against your Teams seat count. Minimum value
    for this setting is 1 month (730h). Must be in the format `300ms` or `2h45m`.
    Valid time units are: `ns`, `us` (or `µs`), `ms`, `s`, `m`, `h`.
    """

    warp_auth_session_duration: str
    """The amount of time that tokens issued for applications will be valid.

    Must be in the format `30m` or `2h45m`. Valid time units are: m, h.
    """


class MfaConfig(TypedDict, total=False):
    """Configures multi-factor authentication (MFA) settings for an organization."""

    allowed_authenticators: List[Literal["totp", "biometrics", "security_key", "ssh_piv_key"]]
    """Lists the MFA methods that users can authenticate with."""

    amr_matching_session_duration: str
    """
    Allows a user to skip MFA via Authentication Method Reference (AMR) matching
    when the AMR claim provided by the IdP the user used to authenticate contains
    "mfa". Must be in minutes (m) or hours (h). Minimum: 0m. Maximum: 720h (30
    days).
    """

    required_aaguids: str
    """Specifies a Cloudflare List of required FIDO2 authenticator device AAGUIDs."""

    session_duration: str
    """Defines the duration of an MFA session.

    Must be in minutes (m) or hours (h). Minimum: 0m. Maximum: 720h (30 days).
    Examples:`5m` or `24h`.
    """


class MfaSSHPivKeyRequirements(TypedDict, total=False):
    """Configures SSH PIV key requirements for MFA using hardware security keys."""

    pin_policy: Literal["never", "once", "always"]
    """Defines when a PIN is required to use the SSH key.

    Valid values: `never` (no PIN required), `once` (PIN required once per session),
    `always` (PIN required for each use).
    """

    require_fips_device: bool
    """
    Requires the SSH PIV key to be stored on a FIPS 140-2 Level 1 or higher
    validated device.
    """

    ssh_key_size: Iterable[Literal[256, 384, 521, 2048, 3072, 4096]]
    """Specifies the allowed SSH key sizes in bits.

    Valid sizes depend on key type. Ed25519 has a fixed key size and does not accept
    this parameter.
    """

    ssh_key_type: List[Literal["ecdsa", "ed25519", "rsa"]]
    """Specifies the allowed SSH key types.

    Valid values are `ecdsa`, `ed25519`, and `rsa`.
    """

    touch_policy: Literal["never", "always", "cached"]
    """Defines when physical touch is required to use the SSH key.

    Valid values: `never` (no touch required), `always` (touch required for each
    use), `cached` (touch cached for 15 seconds).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["WorkspaceOneInputParam"]


class WorkspaceOneInputParam(TypedDict, total=False):
    compliance_status: Required[Literal["compliant", "noncompliant", "unknown"]]
    """Compliance Status."""

    connection_id: Required[str]
    """Posture Integration ID."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ResourceSharingCreateResponse", "Resource"]


class Resource(BaseModel):
    id: str
    """Share Resource identifier."""

    created: datetime
    """When the share was created."""

    meta: object
    """Resource Metadata."""

    modified: datetime
    """When the share was modified."""

    resource_account_id: str
    """Account identifier."""

    resource_id: str
    """Share Resource identifier."""

    resource_type: Literal[
        "custom-ruleset",
        "gateway-policy",
        "gateway-destination-ip",
        "gateway-block-page-settings",
        "gateway-extended-email-matching",
        "idp-federation-grant",
    ]
    """Resource Type."""

    resource_version: int
    """Resource Version."""

    status: Literal["active", "deleting", "deleted"]
    """Resource Status."""


class ResourceSharingCreateResponse(BaseModel):
    id: str
    """Share identifier tag."""

    account_id: str
    """Account identifier."""

    account_name: str
    """The display name of an account."""

    created: datetime
    """When the share was created."""

    modified: datetime
    """When the share was modified."""

    name: str
    """The name of the share."""

    organization_id: str
    """Organization identifier."""

    status: Literal["active", "deleting", "deleted"]

    target_type: Literal["account", "organization"]

    associated_recipient_count: Optional[int] = None
    """The number of recipients in the 'associated' state.

    This field is only included when requested via the 'include_recipient_counts'
    parameter.
    """

    associating_recipient_count: Optional[int] = None
    """The number of recipients in the 'associating' state.

    This field is only included when requested via the 'include_recipient_counts'
    parameter.
    """

    disassociated_recipient_count: Optional[int] = None
    """The number of recipients in the 'disassociated' state.

    This field is only included when requested via the 'include_recipient_counts'
    parameter.
    """

    disassociating_recipient_count: Optional[int] = None
    """The number of recipients in the 'disassociating' state.

    This field is only included when requested via the 'include_recipient_counts'
    parameter.
    """

    kind: Optional[Literal["sent", "received"]] = None

    resources: Optional[List[Resource]] = None
    """A list of resources that are part of the share.

    This field is only included when requested via the 'include_resources'
    parameter.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["DeviceGetResponse", "Account", "User"]


class Account(BaseModel):
    id: Optional[str] = None

    account_type: Optional[str] = None

    name: Optional[str] = None
    """The name of the enrolled account."""


class User(BaseModel):
    id: Optional[str] = None
    """UUID."""

    email: Optional[str] = None
    """The contact email address of the user."""

    name: Optional[str] = None
    """The enrolled device user's name."""


class DeviceGetResponse(BaseModel):
    id: Optional[str] = None
    """Registration ID.

    Equal to Device ID except for accounts which enabled
    [multi-user mode](https://developers.cloudflare.com/cloudflare-one/connections/connect-devices/warp/deployment/mdm-deployment/windows-multiuser/).
    """

    account: Optional[Account] = None

    created: Optional[datetime] = None
    """When the device was created."""

    deleted: Optional[bool] = None
    """True if the device was deleted."""

    device_type: Optional[str] = None

    gateway_device_id: Optional[str] = None

    ip: Optional[str] = None
    """IPv4 or IPv6 address."""

    key: Optional[str] = None
    """The device's public key."""

    key_type: Optional[str] = None
    """Type of the key."""

    last_seen: Optional[datetime] = None
    """When the device last connected to Cloudflare services."""

    mac_address: Optional[str] = None
    """The device mac address."""

    model: Optional[str] = None
    """The device model name."""

    name: Optional[str] = None
    """The device name."""

    os_version: Optional[str] = None
    """The operating system version."""

    serial_number: Optional[str] = None
    """The device serial number."""

    tunnel_type: Optional[str] = None
    """Type of the tunnel connection used."""

    updated: Optional[datetime] = None
    """When the device was updated."""

    user: Optional[User] = None

    version: Optional[str] = None
    """The WARP client version."""
